use crate::entity::benutzer::{self};
use crate::error::AppError;
use crate::model::benutzer::{AktualisiereBenutzer, ErstelleBenutzer};
use anyhow::{Error, Result};
use axum::Json;
use sea_orm::{prelude::*, ActiveValue, DatabaseConnection, DbErr};
use validator::Validate;

pub struct BenutzerRepository {
    db: DatabaseConnection,
}

macro_rules! update_if_some {
    ($active_model:ident, $source:ident, $field:ident) => {
        if let Some(value) = $source.$field {
            $active_model.$field = sea_orm::ActiveValue::Set(value);
        }
    };
}

impl BenutzerRepository {
    pub fn new(db: DatabaseConnection) -> Self {
        Self { db }
    }

    // Benutzer nach ID suchen
    pub async fn find_by_id(&self, id: uuid::Uuid) -> Result<Option<benutzer::Model>, DbErr> {
        benutzer::Entity::find_by_id(id).one(&self.db).await
    }

    // Benutzer erstellen
    pub async fn create_user(
        &self,
        Json(response): Json<ErstelleBenutzer>,
    ) -> Result<Json<benutzer::Model>, AppError> {
        response.validate()?;

        let ersteller_benutzer = benutzer::ActiveModel {
            id: ActiveValue::Set(uuid::Uuid::new_v4()),
            kennung: ActiveValue::Set(response.kennung),
            vorname: ActiveValue::Set(response.vorname),
            nachname: ActiveValue::Set(response.nachname),
            e_mail: ActiveValue::Set(response.email),
            ..Default::default()
        }
        .insert(&self.db)
        .await?;

        Ok(Json(ersteller_benutzer))
    }

    // Benutzer aktualisieren
    pub async fn update_user(
        &self,
        id: uuid::Uuid,
        response: AktualisiereBenutzer,
    ) -> anyhow::Result<benutzer::Model, Error> {
        response.validate()?;

        let benutzer = benutzer::Entity::find_by_id(id)
            .one(&self.db)
            .await?
            .ok_or(DbErr::Custom("User not found.".to_owned()))?;

        let mut active_model: benutzer::ActiveModel = benutzer.into();

        update_if_some!(active_model, response, e_mail);
        update_if_some!(active_model, response, kennung);
        update_if_some!(active_model, response, vorname);
        update_if_some!(active_model, response, nachname);
        update_if_some!(active_model, response, ist_aktiv);

        Ok(active_model.update(&self.db).await?)
    }

    // Benutzer löschen
    pub async fn delete_user(&self, id: uuid::Uuid) -> Result<(), Error> {
        benutzer::Entity::delete_by_id(id).exec(&self.db).await?;
        Ok(())
    }
}
