use chrono::Utc;
use extension::postgres::Type;
use sea_orm::{EnumIter, Iterable};
use sea_orm_migration::{prelude::*, schema::*};

use crate::m20250202_000233_benutzer::Benutzer;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[derive(DeriveIden)]
#[sea_orm(iden = "status_type")]
struct StatusType;

#[derive(DeriveIden, EnumIter)]
pub enum StatusVarianten {
    #[sea_orm(iden = "verfuegbar", string_value = "Verfügbar")]
    Verfuegbar,
    #[sea_orm(iden = "in_benutzung", string_value = "In Benutzung")]
    InBenutzung,
    #[sea_orm(iden = "in_wartung", string_value = "In Wartung")]
    InWartung,
    #[sea_orm(iden = "in_reparatur", string_value = "In Reparatur")]
    InReparatur,
    #[sea_orm(iden = "verloren", string_value = "Verloren")]
    Verloren,
    #[sea_orm(iden = "defekt", string_value = "Defekt")]
    Defekt,
    #[sea_orm(iden = "re_invest", string_value = "ReInvest")]
    ReInvest,
}

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_type(
                Type::create()
                    .as_enum(StatusType)
                    .values(StatusVarianten::iter())
                    .to_owned(),
            )
            .await?;

        manager
            .create_table(
                Table::create()
                    .table(Hardware::Table)
                    .if_not_exists()
                    .col(pk_uuid(Hardware::Id))
                    .col(string(Hardware::Name))
                    .col(string(Hardware::Seriennummer))
                    .col(text(Hardware::Beschreibung))
                    .col(enumeration(
                        Hardware::Status,
                        StatusType,
                        StatusVarianten::iter(),
                    ))
                    .col(uuid(Hardware::BenutzerId))
                    .col(
                        timestamp_with_time_zone(Hardware::ErstelltAm)
                            .not_null()
                            .default(Utc::now()),
                    )
                    .col(
                        timestamp_with_time_zone(Hardware::GeaendertAm)
                            .not_null()
                            .default(Utc::now()),
                    )
                    .foreign_key(
                        ForeignKey::create()
                            .name("fk_hardware_benutzer")
                            .from(Hardware::Table, Hardware::BenutzerId)
                            .to(Benutzer::Table, Benutzer::Id)
                            .on_delete(ForeignKeyAction::SetNull),
                    )
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_hardware_seriennummer")
                    .table(Hardware::Table)
                    .col(Hardware::Seriennummer)
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_hardware_benuzter_id")
                    .table(Hardware::Table)
                    .col(Hardware::BenutzerId)
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(Hardware::Table).to_owned())
            .await?;

        manager
            .drop_type(Type::drop().name(StatusType).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum Hardware {
    Table,
    Id,
    Name,
    Seriennummer,
    Beschreibung,
    Status,
    BenutzerId,
    ErstelltAm,
    GeaendertAm,
}
