use chrono::Utc;
use extension::postgres::Type;
use sea_orm::{EnumIter, Iterable};
use sea_orm_migration::{prelude::*, schema::*};

use crate::m20250202_000233_benutzer::Benutzer;

#[derive(DeriveMigrationName)]
pub struct Migration;

#[derive(DeriveIden)]
#[sea_orm(iden = "event_type")]
struct EventType;

#[derive(DeriveIden, EnumIter)]
pub enum EventVarianten {
    #[sea_orm(iden = "erstellt", string_value = "Erstellt")]
    Erstellt,
    #[sea_orm(iden = "geaendert", string_value = "Geändert")]
    Geaendert,
    #[sea_orm(iden = "geloescht", string_value = "Gelöscht")]
    Geloescht,
}

#[async_trait::async_trait]
impl MigrationTrait for Migration {
    async fn up(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .create_type(
                Type::create()
                    .as_enum(EventType)
                    .values(EventVarianten::iter())
                    .to_owned(),
            )
            .await?;

        manager
            .create_table(
                Table::create()
                    .table(HardwareEvents::Table)
                    .if_not_exists()
                    .col(pk_uuid(HardwareEvents::Id))
                    .col(uuid(HardwareEvents::HardwareId))
                    .col(enumeration(
                        HardwareEvents::Event,
                        EventType,
                        EventVarianten::iter(),
                    ))
                    .col(json(HardwareEvents::Payload))
                    .col(integer(HardwareEvents::Version).not_null().default(1))
                    .col(
                        timestamp_with_time_zone(HardwareEvents::ErstelltAm)
                            .not_null()
                            .default(Utc::now()),
                    )
                    .col(uuid(HardwareEvents::BenutzerId))
                    .foreign_key(
                        ForeignKey::create()
                            .name("fk_hardware_event_benutzer")
                            .from(HardwareEvents::Table, HardwareEvents::BenutzerId)
                            .to(Benutzer::Table, Benutzer::Id)
                            .on_delete(ForeignKeyAction::SetNull),
                    )
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_hardware_events_hardware_id")
                    .table(HardwareEvents::Table)
                    .col(HardwareEvents::HardwareId)
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_hardware_events_erstellt_am")
                    .table(HardwareEvents::Table)
                    .col(HardwareEvents::ErstelltAm)
                    .to_owned(),
            )
            .await?;

        manager
            .create_index(
                Index::create()
                    .unique()
                    .name("idx_hardware_events_event")
                    .table(HardwareEvents::Table)
                    .col(HardwareEvents::Event)
                    .to_owned(),
            )
            .await
    }

    async fn down(&self, manager: &SchemaManager) -> Result<(), DbErr> {
        manager
            .drop_table(Table::drop().table(HardwareEvents::Table).to_owned())
            .await?;

        manager
            .drop_type(Type::drop().name(EventType).to_owned())
            .await
    }
}

#[derive(DeriveIden)]
enum HardwareEvents {
    Table,
    Id,
    HardwareId,
    Event,
    Payload,
    Version,
    ErstelltAm,
    BenutzerId,
}
